/************************************************************************
*
*                          ML8DEMO1.C
*
*           Shows the handling of a MODULAR-4/486 board from a PC
*           with Windows 95
*                                                                      
*					      version      : 1.A                  
*							creation 	 : 16.02.96
*							last change  : see history
*							author       : M.Hcker
*
* 		(c) by SORCUS Computer GmbH Heidelberg 1996
*************************************************************************/

/************************************************************************
*
*	software requirements:
*     - see readme.doc
*
*  hints:
*		- this program is tested with MSVC V.4.0 but it should run with
* 			  other compilers that make 32-bit-code	
*
*     - to compile this example you have to build a 'console application' 
*       because of the use of 'printf'.
*  
*************************************************************************/

/************************************************************************
*  history:
*   	1.A. 16.02.96 (MIH):
*        - creation of the file
*************************************************************************/

#include <conio.h>
#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <windows.h>

#include "ml8bib.h"

#define BLINK_COUNT 20						// number of LED switching
#define MS_WAITING  100					   // delay between switching


void errorhandler(void);

void main(void)
{
	unsigned int	retval;
	unsigned int	w;
	byte           errorclass, errorcode;


   printf("\n***********************************************************\n");
   printf("             DEMO1: makes the LED blinking\n");
   printf("***********************************************************\n");

   /* ml8_bib_startup: this procedure must be called before all other
      ml8_xxx procedures because of library initialisation.
      The used language is C 
   */
   ml8_bib_startup(LANGUAGE_C); 

	/* reset of the board */
   retval = ml8_reset( 0, 						// card number
                  0x380, 					// I/O address
                  0, 						// mode: no interrupts used
                  0, 						// not used 
                  0, 						// not used 
                  * errorhandler, 		// address of error handler called
													//	back by the library
                  "ROM", 					// operating system for the board
                  20);						// timeout


   if (retval>0)                       // if an error occured
   {
	   printf("\nError in 'ml8_reset': \n");
		errorclass = (byte)(retval>>8);
		errorcode  = (byte)(retval);
      printf("errorclass: %02Xh\n", errorclass);
      printf("errorcode:  %02Xh\n", errorcode);
	   ml8_exit();
  	   getch();
      return;
   }

	/*----------- internal LED blinking -------------*/
   printf("\nInternal LED:  ");
   for (w=0; w<BLINK_COUNT; w++)
   {
	   ml8_local_led_on();  	         
	   printf("*");

	   Sleep(MS_WAITING);
	   ml8_local_led_off(); 	 
	   Sleep(MS_WAITING);
   }
   printf("\n");


	/*----------- external LED blinking -------------*/
   printf("\nExternal LED (only if connected):  ");
   for (w=0; w<BLINK_COUNT; w++)
   {
	   ml8_external_led_on();	  
	   printf("*");
	   Sleep(MS_WAITING);
	   ml8_external_led_off();	   
	   Sleep(MS_WAITING);
   }
   printf("\n");

   
   ml8_exit();		           /* use ml8_exit to close the library */
	printf("\n\n<press any key>\n");
	getch();
   return;
}


/********************************************************************
*  errorhandler: 
*     This is a callback function. It is called if there's an error
*     in the library. 
*********************************************************************/
void errorhandler(void)
{
	struct ml8_error_info_type  err;

	ml8_get_error_info( &err);

   printf("\n\n\n************************************************\n");
   printf("************** ERROR OCCURED!!! ****************\n");
   printf("************************************************\n\n");
   printf("errorclass   : %02Xh\n", err.errorclass);
   printf("errorcode    : %02Xh\n", err.errorcode);
	getch();
	exit(1);
}

