/************************************************************************
*
*                          DEMO2.C
*
*           Shows the handling of a MODULAR-4/486 board from a PC
*           with Windows 95: a realtime program is loaded
*                                                                      
*					      version      : 1.A 
*							creation 	 : 16.02.96
*							last change  : see history
*							author       : M.Hcker
*
* 		(c) by SORCUS Computer GmbH Heidelberg 1996
*************************************************************************/

/************************************************************************
*
*	description:
*       The realtime program M8P0300.EXE is loaded to the MODULAR-4/486
*       board. After starting the blink rate of the internal LED can 
*       be modified
*
*	software requirements:
*       see readme.doc
*       The realtime program M8P0300.EXE must be in the actual directory.
*
*  hints:
*		- this program is tested with MSVC V.4.0 but it should run with
* 			  other compilers that make 32-bit-code	
*
*     - to compile this example you have to build a 'console application' 
*       because of the use of 'printf'.
*
*************************************************************************/

/************************************************************************
*  history:
*   	1.A. 16.02.96 (MIH):
*************************************************************************/

#include <conio.h>
#include <stdio.h>
#include <dos.h>
#include <stdlib.h>
#include <windows.h>

#include "ml8bib.h"

#define BLINK_MAX   30000					// maximal blink rate
#define BLINK_MIN   2000 					// minimal blink rate
#define BLINK_STEP  1000					// steps to switch blink rate
#define TASKNUMBER  50

#define RTPROG   "c:\\sorcus\\ml8\\rt\\bc\\beisp\\M8P0300.EXE"
#define OSX      "ROM"

void errorhandler(void);
void ModifyBlinkrate(void);
void SetBrLower(unsigned short *pBr);
void SetBrHigher(unsigned short *pBr);


void main(void)
{
	unsigned int	retval;
	byte           errorclass, errorcode;


   printf("\n***********************************************************\n");
   printf("           DEMO2: loads a program on the board\n");
   printf("***********************************************************\n");

   /* ml8_bib_startup: this procedure must be called before all other
      ml8_xxx procedures because of library initialisation.
      The used language is C 
   */
   ml8_bib_startup(LANGUAGE_C); 

	/* reset of the board */
   retval = ml8_reset( 0, 					// card number
                  0x380, 					// I/O address
                  0, 						// mode: no interrupts used
                  0, 						// not used 
                  0, 						// not used 
                  * errorhandler, 		// address of error handler called
													//	back by the library
                  OSX,  					// operating system for the board
                  20);						// timeout


   if (retval>0)                       // if an error occured
   {
	   printf("\nError in 'ml8_reset': \n");
		errorclass = (byte)(retval>>8);
		errorcode  = (byte)(retval);
      printf("errorclass: %02Xh\n", errorclass);
      printf("errorcode:  %02Xh\n", errorcode);
	   ml8_exit();
  	   getch();
      return;
   }

	/*----------  Loading the realtime program  -----------*/
	printf("\n ** loading the program **\n");
	retval = ml8_transfer_and_install( 
						RTPROG,        		// name of program
						0, 						// dummy, must be set to zero
						TASKNUMBER,				// install program under this 
													//	task number
						0x300, 					// program number
						0, 						// interrupt, not used here
						0x980, 					// flags
						0);						// data size, not needed
	
	if (retval) {								// if error occured
	   printf("\nError in 'ml8_transfer_and_install': \n");
		errorclass = (byte)(retval>>8);
		errorcode  = (byte)(retval);
      printf("errorclass: %02Xh\n", errorclass);
      printf("errorcode:  %02Xh\n", errorcode);
	   ml8_exit();
  	   getch();
      return;
	}

	ml8_wakeup_task(TASKNUMBER);			// start the task

	ModifyBlinkrate();

   ml8_exit();		           /* use ml8_exit to close the library */
   return;
}


/**********************************************************************
*  ModifyBlinkrate:
*     this procedure sets the parameter #1 which is the blinkrate
**********************************************************************/
void ModifyBlinkrate(void)	
{
	char ch = ' ';								// holds keyboard input
	unsigned short br;						// holds blink rate

	printf("\n\n  changing of blinking frequency:\n");
	printf("    <h>: higher frequency\n");
	printf("    <l>: lower frequency\n");
	printf("    <q>: quit program\n\n");

	/* read the actual value of the blink rate (parameter #1) */
	ml8_read_par_block( 	TASKNUMBER, 
								0,					// start parameter 
								sizeof(br),		// number of bytes to read
								&br);				// pointer to write bytes
	do {
		if (kbhit()) {
		   ch = getch();
			if (ch=='l') SetBrHigher(&br);
			if (ch=='h') SetBrLower (&br);
	      /*------- write the new parameter ----------*/
	      ml8_write_par_block( 
	      					TASKNUMBER, 
								1,					// start parameter 
								sizeof(br),		// number of bytes to write
								&br);				// pointer to read bytes
		}
	} while ( ch!='q');
}

/**********************************************************************
*  SetBrHigher:
*     increases a value between BLINK_MAX and BLINK_MIN by step
*     BLINK_STEP
**********************************************************************/
void SetBrHigher(unsigned short *pBr)
{
	*pBr= *pBr+ BLINK_STEP;					// increase Br if not >BLINK_MAX
	if (*pBr>BLINK_MAX) *pBr = BLINK_MAX;
}

/**********************************************************************
*  SetBrLower:
*     decreases a value between BLINK_MAX and BLINK_MIN by step
*     BLINK_STEP
**********************************************************************/
void SetBrLower(unsigned short *pBr)
{
	*pBr= *pBr- BLINK_STEP;					// decrease Br if not <BLINK_MIN
	if (*pBr<BLINK_MIN) *pBr = BLINK_MIN;
}


/********************************************************************
*  errorhandler: 
*     This is a callback function. It is called if there's an error
*     in the library. 
*********************************************************************/
void errorhandler(void)
{
	struct ml8_error_info_type  err;

	ml8_get_error_info( &err);

   printf("\n\n\n************************************************\n");
   printf("************** ERROR OCCURED!!! ****************\n");
   printf("************************************************\n\n");
   printf("errorclass   : %02Xh\n", err.errorclass);
   printf("errorcode    : %02Xh\n", err.errorcode);
	getch();
	exit(1);
}

