/************************************************************************
*
*                          DEMO3.C
*
*           Shows the handling of interrupts of a MODULAR-4/486 board
*           with Windows 95
*
*					      version      : see history
*							creation 	 : 21.12.95
*							last change  : see history
*							author       : M.Haecker
*
* 		(c) by SORCUS Computer GmbH Heidelberg 1995..1996
*************************************************************************/

/************************************************************************
*  description:
*       This sample uses the RT-program M8P0307.EXE. It is loaded onto the
*       MODULAR-4/486 board, is configured and then started. It sends
*       interrupts to the PC which are monitored.
*
*	software requirements:
*       To make an EXE-file you have to link FMLXBIB.LIB and ML8ERRD.C.
*		  To run the application the FMLXBIB.DLL must be in the actual
*       directory or in the systems path (\winnt35\system32)
*       The realtime program M8P0307.EXE must be in the directory 
*       'c:\sorcus\ml8\rt\pascal\beisp'
*
*  hints:
*		- this program is tested with MSVC V.4.1 but it should also run with
* 			  other compilers that make 32-bit-code
*
*     - to compile this example you have to build a 'console application'
*       because of the use of 'printf'.
*
*************************************************************************/

/************************************************************************
*  history:
*   	1.C. 04.08.96 (MIH):
*        - MSVC 4.1
*        - path of real time program changed 
*   	1.B. 26.01.96 (MIH):
*        - forcing alignment = 1 byte for struct 'parameter'
*   	1.A. 21.12.95 (MIH):
*        - creation of the file
*************************************************************************/

#include "windows.h"
#include "conio.h"
#include "stdio.h"
#include "dos.h"
#include "stdlib.h"
#include "string.h"
#include "ml8bib.h"

#define TRUE	1
#define FALSE	0

#define BLINKRATE  0x5000    /* blinkrate for the RT-program M8P0307.EXE */
#define IRQCOUNT   16        /* number of blinking */

#define RTPROG  "c:\\sorcus\\ml8\\rt\\pascal\\beisp\\M8P0307.EXE"
#define OS      "ROM"

void errorhandler(void);
void interrupt_service(void);

ushort	    irq_count= 0;	            /* counter for the interrupts */


void main(void)
{
   ushort	w;
   ushort	pgm,
	         task;
   char		s[80];

   /* the RT-program M8P0307.EXE uses byte alignment; therefore the
      following struct must also use byte byte alignment */
   #pragma pack(push, SORCPACK, 1) 
   struct {
	   byte	status;		               /* state of the RT-program */
	   ushort	blink_rate;             /* blinkrate of the RT-program */
	   byte	led_status;	               /* state of the LED at beginning */
	   byte	led;           	         /* intern or extern LED */
	   ushort	countdown;	            /* number of blinking:
                                        0 = no blinking, 0xFFFF = infinite */
	   byte	with_srq;	               /* send interrupt to PC */
	   ushort	srq_sign;	            /* request message */
	   byte	srq_pending;               /* used by RT-program */
   } parameter;
   #pragma pack(pop, SORCPACK)

   printf(" ***********************************************\n");
   printf("   Sample DEMO3.C (demonstrating PC-Interrupts)\n");
   printf(" ***********************************************\n\n");


   ml8_bib_startup(LANGUAGE_C);        /* language: C */
				                           /* initialisation of the library */

   w = ml8_reset(0,                    // cardnumber
                 0,                    // given by registry
                 1,                    // mode: with interrupts
                 0,                    // given by registry
                 0,                    // not used
                 * errorhandler,       // procedure that is called when an
                                       // error occures
                 OS,                   // operating system for the board
                 20);                  // timeout (* 100 ms)

   if (w>0)                            // if w>0 then an error occured
   {
	   printf("Error at ml8_reset:\n");
      ml8_get_error_msg( s, (byte)(w>>8), (byte)(w&0xFF) );
      printf("Message: %s\nReturn value: %04X\n",s, w);
      ml8_exit();
      exit(1);
   }

   printf("Versions:\n");
   ml8_get_os_version(s);              // get version of OsX
   printf("  OsX Version:    %s\n", s);
   ml8_get_bib_version(s);             // get version of NML8BIB.DLL
   printf("  DLL Version:    %s\n", s);
   ml8_get_drv_version(s);             // get version of MLX-Driver
   printf("  Driver Version: %s\n\n", s);


   ml8_set_service(interrupt_service); // set the interrupt service routine

   pgm = 0x0307;                       // number of the RT-program
   task = 16;                          // task number to install M8P0307.EXE

   w=ml8_transfer_and_install(RTPROG,  // program name
                              0,       // reserved
                              task,    // task number
                              pgm,     // program number
                              0,       // on-board-interrupt (not needed)
                              0x1980,  // flags: wakeup after install,
                                       //        NI-task
                              0);      // data area

   if (w>0)                            // if w>0 then an error occured
   {
    	printf("Error at ml8_transfer_and_install:\n");
      ml8_get_error_msg( s, (byte)(w>>8), (byte)(w&0xFF) );
      printf("Message: %s\nReturn value: %04X\n",s, w);
      ml8_exit();
      exit(1);
   } 

   ml8_suspend_requests();             // don't allow interrupts now


   parameter.status     = 1;           // set parameter struct for RT-program
   parameter.blink_rate = BLINKRATE;
   parameter.led_status = 0;
   parameter.led        = 1;
   parameter.countdown  = IRQCOUNT;
   parameter.with_srq   = TRUE;
   parameter.srq_sign   = 0x0080;
   parameter.srq_pending= FALSE;

                                       // write parameters to board
   ml8_write_par_block(task, 0, sizeof(parameter), &parameter);

   printf("Receiving Interrupts\n\n");
   ml8_allow_requests();               // allow interrupts now

   do {
      Sleep(1);
      //
      // Do anything until all interrupts are received
      //
   } while ( (!kbhit()) && (irq_count<IRQCOUNT) );

   ml8_suspend_requests();             // suspend interrupts
                     
   printf("\nInterrupts received: %d\n\n", irq_count);

   ml8_exit();

   printf("<press any key>");
	getch();
   return;
}

/***************************************************************************
*  Interrupt Service Routine
*     this procedure is called if an interrupt from MODULAR-4/486 board
*     was received
***************************************************************************/
void interrupt_service(void)
{
   ushort message;

   message = ml8_get_message();			// get the message sent by the board
   irq_count++;								// increment interrupz counter
   printf(" Interrupt: #%d; Message: %Xh\n", irq_count, message);
}


/********************************************************************
*  errorhandler:
*     This is a callback function. It is called if there's an error
*     in the library.
*********************************************************************/
void errorhandler(void)
{
   byte   errstr[80];
	struct ml8_error_info_type  err;

	ml8_get_error_info( &err);

   printf("\n\n\n************************************************\n");
   printf("************** ERROR OCCURED!!! ****************\n");
   printf("************************************************\n\n");
	switch (err.errortype) {
      case ERROR_DLL:
	         ml8_get_error_msg(errstr, err.dll.errorclass,
	                  err.dll.errorcode);
	         printf("DLL-Error    : %s\n", errstr);
	         printf("errorclass   : %02Xh\n", err.dll.errorclass);
	         printf("errorcode    : %02Xh\n", err.dll.errorcode);
            break;
      case ERROR_COMMUNICATION:
	         ml8_get_error_msg(errstr, err.dll.errorclass,
	                  err.dll.errorcode);
	         printf("Communication Error: %s\n", errstr);
	         printf("errorclass   : %02Xh\n", err.com.errorclass);
	         printf("errorcode    : %02Xh\n", err.com.errorcode);
            break;
      case ERROR_SYSTEM:
	         printf("System Error: %d(d)\n", err.system.error);
            break;
      case ERROR_DRIVER:
	         printf("Driver Error: \n");
	         printf("expected data length: %d(d)\n", 
	                                       err.drv.dataoutsize);
	         printf("real data length: %d(d)\n", 
	                                       err.drv.retlength);
            break;
      default: printf("unknown error\n");
   }
	getch();
	exit(1);
}

